----------------------------------------------
-- Interfacing a low-cost sonar module with
-- an FPGA 
-- Author: Mike Field <hamster@snap.net.nz>
----------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--use IEEE.NUMERIC_STD.ALL;

entity sonar is
    Port ( clk        : in  STD_LOGIC;
           sonar_trig : out STD_LOGIC;
           sonar_echo : in  STD_LOGIC;
--           anodes     : out STD_LOGIC_VECTOR (3 downto 0);
--           segments   : out STD_LOGIC_VECTOR (6 downto 0));
           Diz7segs   : out std_logic_vector(6 downto 0);
           Unit7segs  : out std_logic_vector(6 downto 0));
end sonar;

architecture Behavioral of sonar is

  component affichage2digits is
    port (  
            I_in8    : in  std_logic_vector(7 downto 0);
				-- Ordre : gfedcba
            Diz7segs,Unit7segs  : out std_logic_vector(6 downto 0)
    );
  end component affichage2digits;
  
    signal count            : std_logic_vector(16 downto 0) := (others => '0');
    signal centimeters      : std_logic_vector(15 downto 0) := (others => '0');
    signal centimeters_ones : std_logic_vector(3 downto 0)  := (others => '0');
    signal centimeters_tens : std_logic_vector(3 downto 0)  := (others => '0');
    signal output_ones      : std_logic_vector(3 downto 0)  := (others => '0');
    signal output_tens      : std_logic_vector(3 downto 0)  := (others => '0');
    signal digit            : std_logic_vector(3 downto 0)  := (others => '0');
    signal echo_last        : std_logic := '0';
    signal echo_synced      : std_logic := '0';
    signal echo_unsynced    : std_logic := '0';
    signal waiting          : std_logic := '0'; 
    signal seven_seg_count  : std_logic_vector(15 downto 0) := (others => '0');
begin

--decode: process(digit)
--    begin
--        case digit is 
--           when "0001" => segments <= "1111001";
--           when "0010" => segments <= "0100100";
--           when "0011" => segments <= "0110000";
--           when "0100" => segments <= "0011001";
--           when "0101" => segments <= "0010010";
--           when "0110" => segments <= "0000010";
--           when "0111" => segments <= "1111000";
--           when "1000" => segments <= "0000000";
--           when "1001" => segments <= "0010000";
--           when "1010" => segments <= "0001000";
--           when "1011" => segments <= "0000011";
--           when "1100" => segments <= "1000110";
--           when "1101" => segments <= "0100001";
--           when "1110" => segments <= "0000110";
--           when "1111" => segments <= "0001110";
--           when others => segments <= "1000000";
--        end case;
--    end process;

--seven_seg: process(clk)
--    begin
--        if rising_edge(clk) then
--            if seven_seg_count(seven_seg_count'high) = '1' then
--                digit <= output_ones;
--                anodes <= "1110";
--            else
--                digit <= output_tens;
--                anodes <= "1101";
--            end if;        
--            seven_seg_count <= seven_seg_count +1; 
--        end if;
--    end process;

  affichage : affichage2digits port map(
       I_in8(3 downto 0) =>  output_ones,
		 I_in8(7 downto 4) =>  output_tens,
       Diz7segs => Diz7segs,
		 Unit7segs => Unit7segs 
  );
    
process(clk)
    begin
        if rising_edge(clk) then
            if waiting = '0' then
            --    if count = 1000 then -- Assumes 100MHz
				   if count = 500 then -- Assumes 50MHz
                   -- After 10us then go into waiting mode
                   sonar_trig <= '0';
                   waiting    <= '1';
                   count       <= (others => '0');
                else
                   sonar_trig <= '1';
                   count <= count+1;
                end if;
            elsif echo_last = '0' and echo_synced = '1' then
                -- Seen rising edge - start count
                count       <= (others => '0');
                centimeters <= (others => '0');
                centimeters_ones <= (others => '0');
                centimeters_tens <= (others => '0');
            elsif echo_last = '1' and echo_synced = '0' then
                -- Seen falling edge, so capture count
                output_ones <= centimeters_ones; 
                output_tens <= centimeters_tens; 
 --           elsif count = 2900*2 -1 then
             elsif count = 1450*2 -1 then
                -- advance the counter
                if centimeters_ones = 9 then
                    centimeters_ones <= (others => '0');
                    centimeters_tens <= centimeters_tens + 1;
                else
                    centimeters_ones <= centimeters_ones + 1;
                end if;
                centimeters <= centimeters + 1;
                count <= (others => '0');
                if centimeters = 3448 then
                    -- time out - send another pulse
                    waiting <= '0';
                end if;
            else
                count <= count + 1;                
            end if;

            echo_last     <= echo_synced;
            echo_synced   <= echo_unsynced;
            echo_unsynced <= sonar_echo;
        end if;
        
    end process;
end Behavioral;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
-- zero pour allumer les LEDs
entity transcod7segs is
    port (  
            I_in4    : in  std_logic_vector(3 downto 0);
				-- Ordre : gfedcba
            Q_7segs  : out std_logic_vector(6 downto 0)
    );
end transcod7segs;

architecture arch of transcod7segs is begin
  with I_in4 select
    Q_7SEGS <= "1000000" when "0000",
               "1111001" when "0001",
               "0100100" when "0010",
               "0110000" when "0011",
               "0011001" when X"4",
               "0010010" when X"5",
               "0000010" when X"6",
               "1111000" when X"7",
               "0000000" when X"8",
               "0010000" when X"9",
               "0001000" when X"A",
               "0000011" when X"B",
               "1000110" when X"C",
               "0100001" when X"D",
               "0000110" when X"E",
               "0001110" when others; --X"F"
end arch;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
-- zero pour allumer les segments
entity affichage2digits is
    port (  
            I_in8    : in  std_logic_vector(7 downto 0);
				-- Ordre : gfedcba
            Diz7segs,Unit7segs  : out std_logic_vector(6 downto 0)
    );
end affichage2digits;

architecture arch_affichage2digits of affichage2digits is
component transcod7segs is
    port (  
            I_in4    : in  std_logic_vector(3 downto 0);
				-- Ordre : gfedcba
            Q_7segs  : out std_logic_vector(6 downto 0)
    );
end component transcod7segs;
 
begin
  unite : transcod7segs port map (
     I_in4 => I_in8(3 downto 0),
	  Q_7segs => Unit7segs
	  );
  dizaine : transcod7segs port map (
     I_in4 => I_in8(7 downto 4),
	  Q_7segs => Diz7segs
	  );
end arch_affichage2digits;